@@ -68,7 +68,12 @@ insilico.fit <- function(data, data.type = c("WHO2012", "WHO2016")[1],isNumeric
   args <- as.list(match.call())
   if(!is.null(args$length.sim)){
   	Nsim <- args$length.sim
-  	cat("length.sim argument is replaced with Nsim argument, will remove in later versions.\n")
+  	message("length.sim argument is replaced with Nsim argument, will remove in later versions.\n")
+  }
+
+  if(data.type == "WHO2016" & "i183o" %in% colnames(data)){
+  	colnames(data)[which(colnames(data) == "i183o")] <- "i183a"
+  	message("Due to the inconsistent names in the early version of InterVA5, the indicator 'i183o' has been renamed as 'i183a'.")
   }
 
 ##----------------------------------------------------------##
@@ -320,8 +325,8 @@ datacheck.interVAJava <- function(data, obj, warning.write){
 		# this has been updated to correspond to the 4.03 version probbase which contains minor changes from before.
 		data("probbase3", envir = environment())
 		probbase <- get("probbase3", envir  = environment())
-
-		
+		# fix symptom name that has been changed in 4.03
+		probbase[which(probbase=="sk_les")] <- "skin_les"
 		# THIS STEP CHECKS HOW 'STRUCTURED MISSING' ARE IMPLEMENTED, SEE VIGNETT FOR DETAILS.
 		# 1. if no symptoms should be checked to be missing...
 		# zero_to_missing_list <- 0
@@ -376,22 +381,25 @@ datacheck.interVA5 <- function(data, obj, warning.write){
 
 		data.num <- matrix(0, dim(data)[1], dim(data)[2] - 1)
 		for(j in 2:dim(data)[2]){
-			data.num[which(toupper(data[, j]) == "Y"), j - 1] <- 1			
-			data.num[which(data[, j] == "."), j - 1] <- NA
+			data.num[which(toupper(data[, j]) == "Y"), j - 1] <- 1
+			data.num[which(data[, j] %in% c("N", "n", "Y", "y") == FALSE), j - 1] <- NA
 		}
+		# to be consistent with InterVA5, adding first column
+		data.num <- cbind(NA, data.num)
+		checked <- data.num
 
 		# S <- dim(probbaseV5)[1]
 		# subst.vector <- rep(NA, length=S)
         # subst.vector[probbaseV5[,6]=="N"] <- 0
         # subst.vector[probbaseV5[,6]=="Y"] <- 1
 
-		checked <- data.num
 		warning <- NULL
 		for(i in 1:dim(data)[1]){
-			tmp <- InterVA5::DataCheck5(data.num[i,], id=data[i,1], probbaseV5=probbaseV5, write=warning.write)
+			tmp <- InterVA5::DataCheck5(data.num[i,], id=data[i,1], probbaseV5=probbaseV5, InSilico_check = TRUE, write=warning.write)
 	        input.current <- tmp$Output
 	        warning[[i]] <- rbind(tmp$firstPass, tmp$secondPass)
 	        checked[i, ] <- input.current
+	        if(i %% 10 == 0) cat(".")
 	    }
 
 		return(list(checked=checked, warning = warning))
@@ -499,12 +507,12 @@ removeExt <- function(data, prob.orig, is.Numeric, subpop, subpop_order_list, ex
 				ext.csmf = ext.csmf))
 }
 
-removeExtV5 <- function(data, prob.orig, is.Numeric, subpop, subpop_order_list, external.causes, external.symps){
+removeExtV5 <- function(data, prob.orig, csmf.orig, is.Numeric, subpop, subpop_order_list, external.causes, external.symps){
 ###########################################################
 # function to remove external causes/symps and assign deterministic deaths
 # @param:
 #	   data
-#      prob.orig: directly from InterVA4
+#      prob.orig: directly from InterVA5
 # @values:
 #	   data: after removing external death
 #	   prob.orig: after deleting external symptoms
@@ -518,7 +526,7 @@ removeExtV5 <- function(data, prob.orig, is.Numeric, subpop, subpop_order_list,
 		neg <- 0
 		pos <- 1
 	}else{
-		neg <- ""
+		neg <- "N"
 		pos <- "Y"
 	}
 	ext.where <- which(apply(extData, 1, function(x){
@@ -526,33 +534,46 @@ removeExtV5 <- function(data, prob.orig, is.Numeric, subpop, subpop_order_list,
 	extData <- as.matrix(extData[ext.where, ])
 	ext.id <- data[ext.where, 1]
 	ext.sub <- subpop[ext.where]
+	probsub <- change.inter(prob.orig[external.symps, external.causes])
+	csmfsub <- change.inter(csmf.orig[external.causes])
+
 	# a smaller scale datacheck for external causes
-	extData[which(extData[,2] == pos), c(3, 5, 7, 10, 11, 13)] <- neg
-	extData[which(extData[,3] == pos), c(2, 5, 7, 10, 11, 13)] <- neg
-	extData[which(extData[,4] == pos), c(5, 7, 11, 13)] <- neg
-	extData[which(extData[,5] == pos), c(2,3,4,8,9,15)] <- neg
-	extData[which(extData[,6] == pos), c(5,9,13,15)] <- neg
-	extData[which(extData[,7] == pos), c(2,3,5,6,8,9,11,15)] <- neg
-	extData[which(extData[,8] == pos), c(5,7,9,11,15)] <- neg
-	extData[which(extData[,9] == pos),c(5,7,8,11,12,13,14,15)] <- neg
-	extData[which(extData[,15] == pos),c(5,7,8)] <- neg
-	extData[which(extData[,18] == pos),c(15)] <- neg
-	extData[which(extData[,19] == pos),c(3,15)] <- neg
+	# extData[which(extData[,2] == pos), c(3, 5, 7, 10, 11, 13)] <- neg
+	# extData[which(extData[,3] == pos), c(2, 5, 7, 10, 11, 13)] <- neg
+	# extData[which(extData[,4] == pos), c(5, 7, 11, 13)] <- neg
+	# extData[which(extData[,5] == pos), c(2,3,4,8,9,15)] <- neg
+	# extData[which(extData[,6] == pos), c(5,9,13,15)] <- neg
+	# extData[which(extData[,7] == pos), c(2,3,5,6,8,9,11,15)] <- neg
+	# extData[which(extData[,8] == pos), c(5,7,9,11,15)] <- neg
+	# extData[which(extData[,9] == pos),c(5,7,8,11,12,13,14,15)] <- neg
+	# extData[which(extData[,15] == pos),c(5,7,8)] <- neg
+	# extData[which(extData[,18] == pos),c(15)] <- neg
+	# extData[which(extData[,19] == pos),c(3,15)] <- neg
 	
 	# initialize with all "unspecified ext causes"
-	ext.cod <- rep(extCauses[11], length(ext.id))
-	# begin checking symptoms
-	ext.cod[which(extData[,2] == pos)] <- extCauses[1]
-	ext.cod[which(extData[,3] == pos)] <- extCauses[2]
-	ext.cod[which(extData[,4] == pos)] <- extCauses[3]
-	ext.cod[which(extData[,5] == pos)] <- extCauses[7]
-	ext.cod[which(extData[,6] == pos)] <- extCauses[4]
-	ext.cod[which(extData[,9] == pos)] <- extCauses[5]
-	ext.cod[which(extData[,7] == pos)] <- extCauses[6]
-	ext.cod[which(extData[,10] == pos)] <- extCauses[9]
-	ext.cod[which(extData[,19] == pos)] <- extCauses[9]
-	ext.cod[which(extData[,15] == pos)] <- extCauses[10]
-	ext.cod[which(extData[,18] == pos)] <- extCauses[8]
+	# ext.cod <- rep(extCauses[11], length(ext.id))
+	# # begin checking symptoms
+	# ext.cod[which(extData[,2] == pos)] <- extCauses[1]
+	# ext.cod[which(extData[,3] == pos)] <- extCauses[2]
+	# ext.cod[which(extData[,4] == pos)] <- extCauses[3]
+	# ext.cod[which(extData[,5] == pos)] <- extCauses[7]
+	# ext.cod[which(extData[,6] == pos)] <- extCauses[4]
+	# ext.cod[which(extData[,9] == pos)] <- extCauses[5]
+	# ext.cod[which(extData[,7] == pos)] <- extCauses[6]
+	# ext.cod[which(extData[,10] == pos)] <- extCauses[9]
+	# ext.cod[which(extData[,19] == pos)] <- extCauses[9]
+	# ext.cod[which(extData[,15] == pos)] <- extCauses[10]
+	# ext.cod[which(extData[,18] == pos)] <- extCauses[8]
+
+	probs <- matrix(1, dim(extData)[1], length(external.causes))
+	for(i in 1:dim(extData)[1]){
+		for(j in 1:length(external.causes)){
+			probs[i, j] <- csmfsub[j] * prod(probsub[which(extData[i,] == pos), j])
+		}
+		probs[i, ] <- probs[i, ] / sum(probs[i, ])
+	}
+	ext.prob <- probs
+
 
 	# delete death confirmed external
 	if(length(ext.where) > 0){
@@ -566,20 +587,13 @@ removeExtV5 <- function(data, prob.orig, is.Numeric, subpop, subpop_order_list,
 		ext.csmf <- vector("list", length(subpop_order_list))
 		for(i in 1:length(ext.csmf)){
 			ext.csmf[[i]] <- rep(0, length(extCauses))
-			ext.cod.temp <- ext.cod[which(ext.sub == subpop_order_list[i])]
-			if(!is.null(ext.cod.temp)){
-				for(j in 1:length(extCauses)){
-					ext.csmf[[i]][j] <- length(which(ext.cod.temp == extCauses[j]))
-				}
-				ext.csmf[[i]] <- ext.csmf[[i]]/length(which(subpop == subpop_order_list[i]))		
+			ext.prob.temp <- ext.prob[which(ext.sub == subpop_order_list[i]), , drop = FALSE]
+			if(!is.null(ext.prob.temp)){
+					ext.csmf[[i]] <- apply(ext.prob.temp, 2, mean) * dim(ext.prob.temp)[1] / length(which(subpop == subpop_order_list[i]))		
 			}
 		}
 	}else{
-		ext.csmf <- rep(0, length(extCauses))
-		for(i in 1:length(extCauses)){
-			ext.csmf[i] <- length(which(ext.cod == extCauses[i]))
-		}
-		ext.csmf <- ext.csmf/N.all		
+		ext.csmf <- apply(ext.prob, 2, mean) * length(ext.id) / N.all		
 	}
 	if(length(ext.where) > 0) subpop <- subpop[-ext.where]
 	return(list(data = data, 
@@ -587,7 +601,8 @@ removeExtV5 <- function(data, prob.orig, is.Numeric, subpop, subpop_order_list,
 				prob.orig = prob.orig, 
 				ext.sub  = ext.sub,
 				ext.id = ext.id, 
-				ext.cod = ext.cod,
+				ext.prob = ext.prob,
+				ext.cod = NULL,
 				ext.csmf = ext.csmf))
 }
 
@@ -673,7 +688,8 @@ ParseResult <- function(N_sub.j, C.j, S.j, N_level.j, pool.j, fit){
 			data[, i] <- as.character(data[, i])
 			data[data[,i]=="n", i] <- ""
 			data[data[,i]=="N", i] <- ""
-			data[data[,i]=="-", i] <- "."
+			misstmp <- which(data[,i] %in% c("Y", "y", "N", "n") == FALSE)
+			if(length(misstmp) > 0) data[misstmp, i] <- "."
 		}
 	} 
 	if(no.is.missing){
@@ -723,6 +739,8 @@ ParseResult <- function(N_sub.j, C.j, S.j, N_level.j, pool.j, fit){
 	if(data.type == "WHO2012"){
 		data("probbase3", envir = environment())
 		probbase<- get("probbase3", envir  = environment())
+		# fix symptom name that has been changed in 4.03
+		probbase[which(probbase=="sk_les")] <- "skin_les"
 		data("causetext", envir = environment())
 		causetext<- get("causetext", envir  = environment())		
 	}else{
@@ -888,24 +906,26 @@ ParseResult <- function(N_sub.j, C.j, S.j, N_level.j, pool.j, fit){
 
 
   	if(datacheck){
-		cat("Performing data consistency check...\n")
+		message("Performing data consistency check...\n")
 		if(data.type == "WHO2012"){
 			# code missing as -1
 			checked <- datacheck.interVAJava(data, obj, warning.write)
 			warning <- NULL
+			offset <- 0
 		}else{
 			# code missing as NA
 			checked <- datacheck.interVA5(data, obj, warning.write)
 			warning <- checked$warning
 			checked <- checked$checked
+			offset <- 1
 		}
-		cat("Data check finished.\n")
+		message("Data check finished.\n")
 		## update in data with missing, nothing is updated into missing, so only changing Y and N
 		for(i in 1:(dim(data)[2]-1)){
-			data[which(checked[, i] == 1), i+1] <- "Y"
-			data[which(checked[, i] == 0), i+1] <- ""
-			data[which(checked[, i] == -1), i+1] <- "." # for 2012
-			data[which(is.na(checked[, i])), i+1] <- "." # for 2016
+			data[which(checked[, i+offset] == 1), i+1] <- "Y"
+			data[which(checked[, i+offset] == 0), i+1] <- ""
+			data[which(checked[, i+offset] == -1), i+1] <- "." # for 2012
+			data[which(is.na(checked[, i+offset])), i+1] <- "." # for 2016
 		}	
   	}else{
   		warning <- NULL
@@ -917,7 +937,8 @@ ParseResult <- function(N_sub.j, C.j, S.j, N_level.j, pool.j, fit){
 
   		
   		}else{
-			externals <- removeExtV5(data,prob.orig, isNumeric, subpop, subpop_order_list, external.causes, external.symps)
+  			csmf.orig <- probbase[1, -(1:20)]
+			externals <- removeExtV5(data,prob.orig, csmf.orig, isNumeric, subpop, subpop_order_list, external.causes, external.symps)
   		}
   		data <- externals$data
   		subpop <- externals$subpop
@@ -951,7 +972,7 @@ ParseResult <- function(N_sub.j, C.j, S.j, N_level.j, pool.j, fit){
    	## check interVA rules, ignoring missing at this step,
    	## since missing could be rewritten 
    	if((!datacheck.missing) && datacheck){
-		cat("check missing after removing symptoms are disabled...\n")
+		message("check missing after removing symptoms are disabled...\n")
 	}
 
 
@@ -1123,10 +1144,10 @@ ParseResult <- function(N_sub.j, C.j, S.j, N_level.j, pool.j, fit){
 		}
 		
 		if(external.sep){
-			cat(paste(length(matchid), 
+			message(paste(length(matchid), 
 				"deaths found known physician coding after removing deaths from external causes.\n"))			
 		}else{
-			cat(paste(length(matchid), 
+			message(paste(length(matchid), 
  				"deaths found known physician coding.\n"))
 		}
 	}else{
@@ -1388,8 +1409,8 @@ ParseResult <- function(N_sub.j, C.j, S.j, N_level.j, pool.j, fit){
 			burn.j <- as.integer(0)
 			keepProb.j <- !updateCondProb
 
-			cat(paste("Not all causes with CSMF >", conv.csmf, "are convergent.\n"))
-    		cat(paste("Increase chain length with another", N_gibbs.j, "iterations\n"))
+			message(paste("Not all causes with CSMF >", conv.csmf, "are convergent.\n"))
+    		message(paste("Increase chain length with another", N_gibbs.j, "iterations\n"))
     		obj <- .jnew("sampler/InsilicoSampler2")
     		ins  <- .jcall(obj, "[D", "Fit", 
 						dimensions.j, 
@@ -1419,7 +1440,7 @@ ParseResult <- function(N_sub.j, C.j, S.j, N_level.j, pool.j, fit){
     }
     ## if still not convergent 
     if(!conv){
-    	cat(paste("Not all causes with CSMF >", conv.csmf, "are convergent.\n",
+    	message(paste("Not all causes with CSMF >", conv.csmf, "are convergent.\n",
     			  "Please check using csmf.diag() for more information.\n"))
     }
     csmf.sub  <- results$csmf.sub 
@@ -1464,7 +1485,8 @@ ParseResult <- function(N_sub.j, C.j, S.j, N_level.j, pool.j, fit){
 				    						  	   dim(temp)[1],
 				    						  	   length(external.causes), 
 				    						  	   byrow = TRUE), 
-				    						temp[, ext1:C.j])	
+				    						temp[, ext1:C.j])
+				csmf.sub.all[[j]][is.nan(csmf.sub.all[[j]])] <- 0    							
     		}
     		csmf.sub <- csmf.sub.all
     	##
@@ -1483,9 +1505,17 @@ ParseResult <- function(N_sub.j, C.j, S.j, N_level.j, pool.j, fit){
     	
     	p.indiv.ext <- matrix(0, nrow = length(externals$ext.id), ncol = C.j + length(external.causes) )
     	if(length(externals$ext.id) > 0){
-	    	for(i in 1:length(externals$ext.id)){p.indiv.ext[i, externals$ext.cod[i]] <- 1}    		
+    		# WHO 2012
+    		if(is.null(externals$ext.prob)){
+	    		for(i in 1:length(externals$ext.id)){p.indiv.ext[i, externals$ext.cod[i]] <- 1}    
+    		# WHO 2016	
+    		}else{
+				p.indiv.ext[, external.causes] <- externals$ext.prob
+    		}
+		
     	}
     	p.indiv <- rbind(p.indiv, p.indiv.ext) 
+    	p.indiv[is.nan(p.indiv)] <- 0
     	id <- c(id, externals$ext.id)
     	subpop <- c(subpop, externals$ext.sub)
    }
@@ -1563,7 +1593,8 @@ out <- list(
 		indiv.CI = indiv.CI, 
 		is.customized = customization.dev,
 		errors = errorlog,
-		warning = warning)
+		warning = warning,
+		data.type = data.type)
 
 # get also individual probabilities
 if(!is.null(indiv.CI)){
@@ -1572,7 +1603,6 @@ if(!is.null(indiv.CI)){
 	out$indiv.prob.upper <- indiv$upper
 	out$indiv.prob.lower <- indiv$lower			
 }
-
 class(out) <- "insilico"
 return(out)  	
 } 
